import debounce from 'debounce';
import BaseForm from 'controllers/base_form_controller';

export default class extends BaseForm {
  static targets = ['mySelection', 'inputSearch', 'tagsetRadio', 'loader', 'fileInput', 'regroupButton'];

  connect() {
    super.connect();

    this.addRadioListeners();

    if (this.hasFileInputTarget) {
      this.fileInputTarget.addEventListener('updatedFiles', this.fileInputChanged.bind(this));
    }

    this.inputSearchTarget.addEventListener('keyup', debounce(this.handleKeyup.bind(this), 300));
  }

  disconnect() {
    super.disconnect();

    this.removeRadioListeners();

    if (this.hasFileInputTarget) {
      this.fileInputTarget.removeEventListener('updatedFiles', this.updateSubmitVisibilityHandler);
    }

    this.inputSearchTarget.removeEventListener('keyup', this.handleKeyup.bind(this));
  }

  addRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.addEventListener('change', this.updateSubmitVisibilityHandler);
    });
  }

  removeRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.removeEventListener('change', this.updateSubmitVisibilityHandler);
    });
  }

  fileInputChanged() {
    this.updateSubmitVisibility();
    this.updateRegroupButtonVisbility();
    this.updateAddActionVisibility();
  }

  updateRegroupButtonVisbility() {
    if (this.hasFileInputTarget && this.hasRegroupButtonTarget) {
      this.regroupButtonTarget.classList.toggle('hidden', this.fileInputTarget.files.length <= 1);
    }
  }

  calculateShouldShowSubmit() {
    const allFieldsFilled = Array.from(this.requiredFields).every(el => this.isNotEmptyField(el));
    const isAddActionValid = this.addActionTarget.checked ? !!this.nextActionTarget.value : true;
    const fileOrRadioRequired = this.hasFileInputTarget ? this.fileInputTarget.files.length > 0 : this.tagsetRadioTargets.some(radio => radio.checked);
    return allFieldsFilled && isAddActionValid && fileOrRadioRequired;
  }

  handleKeyup(event) {
    const query = event.target.value;

    if (query.length > 2) {
      this.search(query);
    }
  }

  search(query) {
    const url = `/spirit_hub/tagsets/search_directories?query=${encodeURIComponent(query)}`;
    this.loaderTarget.classList.remove('hidden');

    fetch(url, { headers: { Accept: 'text/vnd.turbo-stream.html' } })
      .then(response => response.text())
      .then(html => {
        this.removeRadioListeners();
        Turbo.renderStreamMessage(html);
        requestAnimationFrame(() => {
          this.addRadioListeners();
          this.updateSubmitVisibility.bind(this);
          this.loaderTarget.classList.add('hidden');
        });
      })
      .catch(error => console.error('Error:', error));
  }

  tagsetChanged(event) {
    const selectedValue = event.target.value;
    const isChecked = event.target.checked;
    const matchingDivs = this.element.querySelectorAll(`div[data-tagset-id="${selectedValue}"]`);

    if (isChecked) {
      matchingDivs.forEach(div => div.classList.remove('hidden'));
    }

    this.updateRegroupButtonDisabled(false);
  }

  completenessChanged() {
    this.updateRegroupButtonDisabled(true);
  }

  updateRegroupButtonDisabled(disabled) {
    if (!this.hasRegroupButtonTarget) {
      return;
    }

    const input = this.regroupButtonTarget.querySelector('input[type="checkbox"]');

    if (disabled) {
      input.checked = true;
      this.updateAddActionVisibility();
    }

    this.regroupButtonTarget.classList.toggle('cursor-not-allowed', disabled);
    this.regroupButtonTarget.classList.toggle('opacity-50', disabled);
  }

  updateAddActionVisibility() {
    if (!this.hasRegroupButtonTarget || !this.hasAddActionTarget || !this.hasFileInputTarget) {
      return;
    }

    const input = this.regroupButtonTarget.querySelector('input[type="checkbox"]');
    const showAddAction = this.fileInputTarget.files.length < 2 || input.checked;
    const addActionContainer = this.addActionTarget.parentElement.parentElement.parentElement;
    const nextActionContainer = addActionContainer.querySelector('div[data-checkbox-hide-target="item"]');

    if (!showAddAction) {
      this.addActionTarget.checked = false;
      nextActionContainer.classList.add('hidden');
    }

    addActionContainer.classList.toggle('hidden', !showAddAction);
    this.updateSubmitContent();
  }
}
