# frozen_string_literal: true

module SpiritHub
  class OperationsController < ApplicationController
    before_action :set_seach_params, only: %i[index sort]
    before_action :find_models, only: %i[index sort]
    before_action :find_model, only: %i[edit update destroy]
    after_action :verify_policy_scoped, except: %i[new create]

    include Datatablable

    def index
      authorize Operation

      respond_to do |format|
        format.html
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:model_table, partial: 'spirit_hub/operations/table')
          ]
        end
      end
    end

    def sort
      authorize Operation

      render partial: 'spirit_hub/operations/table'
    end

    def new
      authorize Operation

      model = SpiritHub::Operation.new

      render turbo_stream: [generate_operation_form(model, false)]
    end

    def create
      authorize Operation

      model = SpiritHub::Operation.new(operation_params)
      if model.save
        model.generate_operation_tree(@user)

        set_user_favorites

        @turbo_actions = []
        @turbo_actions << turbo_stream.append(:operations_tbody, partial: 'spirit_hub/operations/row', locals: { model: })

        flash[:notice] = 'Projet créé'
        @turbo_actions << turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        @turbo_actions << turbo_stream.redirect_to(spirit_hub_directory_path(id: model.root_tagset_id))
        render turbo_stream: @turbo_actions
      else
        render turbo_stream: [generate_operation_form(model, false)]
      end
    end

    def edit
      authorize @model

      render turbo_stream: [generate_operation_form(@model, true)]
    end

    def update
      authorize @model

      if @model.update(operation_params)
        set_user_favorites

        @turbo_actions = []
        @turbo_actions << turbo_stream.replace("operation_#{@model.id}", partial: 'spirit_hub/operations/row', locals: { model: @model })

        flash.now[:notice] = 'Projet modifié'
        @turbo_actions << turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')

        render turbo_stream: @turbo_actions
      else
        render turbo_stream: [generate_operation_form(@model, true)]
      end
    end

    def destroy
      authorize @model

      @model.current_user_id = @user.id
      @model.destroy

      @turbo_actions = []
      @turbo_actions << turbo_stream.remove("operation_#{@model.id}")

      flash.now[:notice] = 'Projet supprimé'
      @turbo_actions << turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')

      render turbo_stream: @turbo_actions
    end

    private

    def operation_params
      params[:operation].permit(%i[title
                                   code
                                   address
                                   postal_code
                                   city
                                   agency
                                   type
                                   start_date
                                   end_date
                                   responsible
                                   current_user_id
                                   classement_plan_id])
    end

    def generate_operation_form(model, edit)
      responsible_users = SpiritHub::ActorFormatter.actor_ids_to_select(model.responsible)
      current_users = SpiritHub::ActorFormatter.actor_ids_to_select(model.current_user_id)
      valid_docs = SpiritHub::OperationTreeModel.find_valid_docs.map { |doc| [doc.title, doc.id] }
      turbo_stream.update(:modal, partial: 'spirit_hub/operations/form', locals: { model:, edit:, responsible_users:, current_users:, valid_docs: })
    end

    def default_columns
      {
        code: { label: 'Code', sortable: true },
        title: { label: 'Libellé', sortable: true },
        city: { label: 'Ville', sortable: true },
        postal_code: { label: 'Code postal', sortable: true },
        status: { label: 'Statut', sortable: true },
        agency: { label: 'Agence', sortable: true },
        type: { label: 'Type', sortable: true }
      }.freeze
    end

    def find_models
      scope = policy_scope(Operation)
      scope = scope.where(title: /#{@search_params[:operation_title]}/i) if @search_params && @search_params[:operation_title]
      scope = scope.where(agency: /#{@search_params[:agence_name]}/i) if @search_params && @search_params[:agence_name]
      @models = find_models_page(scope, default_sort_by: 'title')
      @columns = default_columns
    end

    def set_seach_params
      @search_params = params[:search_params]&.permit(:operation_title, :agence_name)&.to_h
    end

    def find_model
      @model = policy_scope(Operation).find_by!(id: params[:id])
    end
  end
end
