# frozen_string_literal: true

module SpiritHub
  class JournalEntry < ApplicationRecord
    CREATE_CODES = [
      'DOC_CREATED',
      'DOC_CREATED_FROM_TEMPLATE', # not used so far
      'DOC_ADDED_TO_COMPLETENESS',
      'OPERATION_CREATED',
      'DIRECTORY_CREATED'
    ].freeze

    DELETE_CODES = %w[
      DOC_DELETED
      DIRECTORY_DELETED
    ].freeze

    SHARE_CODES = %w[
      DOC_SHARED
      MULTIPLE_DOC_SHARED
      SHARE_LINK_CREATED
      COMMENT_ADDED_TO_LINK
    ].freeze

    CLASSIFY_CODES = [
      'DOC_CLASSIFIED'
    ].freeze

    EDIT_CODES = %w[
      DOC_EDITED_IN_OFFICE_ONLINE
      DIRECTORY_INFORMATION_UPDATED
    ].freeze

    PROCESS_CODES = %w[
      DOC_ASSIGNED
      DOC_SIGNED
      DOC_TO_SIGN
      DOC_RESPONDED
    ].freeze

    SENT_TO_CODES = %w[
      DOC_VALIDATED
      DOC_REFUSED
      DOC_SENT_TO_SHARE
      DOC_SENT_TO_VALIDATE
      DOC_SENT_TO_COMPLETE
      DOC_SENT_TO_ANSWER
      DOC_SENT_TO_SIGN
    ].freeze

    field :code, type: String
    field :action, type: String

    belongs_to :user, optional: true
    belongs_to :tagset, optional: true

    class << self
      %i[create delete share classify edit process sent_to].each do |log_type|
        define_method("#{log_type}_action_log") do |code, opts = {}|
          raise "#{log_type} action code missing" if code.nil?
          raise "#{log_type} action code invalid" unless "SpiritHub::JournalEntry::#{log_type.upcase}_CODES".constantize.include?(code)

          action_log(code, opts)
        end
      end

      private

      def action_log(code, opts)
        tagset = opts[:tagset]
        user = opts[:user]
        recipients = SpiritHub::ActorFormatter.actor_ids_to_sentence(opts[:recipients], without_indicator: true)

        message_options = {
          title: opts[:title].presence || tagset&.title,
          username: user&.full_name || user&.email,
          recipients:,
          link: opts[:link],
          recipient: opts[:recipient],
          directory: opts[:directory]
        }

        action_msg = [I18n.t(code, scope: 'journal_entries.action', **message_options)]
        action_msg << opts[:action_msg].presence

        create(
          code:,
          tagset_id: tagset&.id || opts[:tagset_id],
          user_id: user&.id || opts[:user_id],
          action: action_msg.compact.join(' : ')
        )
      end
    end
  end
end
