# frozen_string_literal: true

module SpiritHub
  class HubdocCycle
    class << self
      def import_cycle_from_json(*)
        new(*).import_cycle_from_json
      end

      def default_cycle
        Cycle.find_by(key: 'hubdoc')
      end

      def find_state_by_key(key)
        State.find_by(key:, cycle_id: default_cycle.id)
      end

      def change_state!(model, key, user)
        change_state(model, key, user)
        model.save!
      end

      def change_state(model, key, user)
        model.last_current_user_name = user&.full_name
        model.last_current_user_id = user&.id
        model.state_id = SpiritHub::HubdocCycle.find_state_by_key(key).id
      end

      def possible_current_states
        states_ids = State.where(cycle_id: default_cycle.id).order_by(position: :asc).pluck(:id)
        transitions_ids = Transition.where('$or' => [
                                             { :to_state_id.in => states_ids },
                                             { :from_state_id.in => states_ids }
                                           ]).distinct(:trigger_ids).compact
        possible_current_states = Trigger.where(:id.in => transitions_ids).distinct(:set_state).compact
        directory_states = ['À completer', 'Complet']

        (directory_states + possible_current_states).uniq
      end

      def all_next_possible_states(current_state)
        default_cycle = Cycle.find_by(key: 'hubdoc')
        ids_to_remove = default_cycle.initial_state_ids.to_a
        ids_to_remove << current_state.id if current_state.present?
        states = State.where(cycle_id: default_cycle.id, :id.nin => ids_to_remove).order_by(position: :asc)

        states = if current_state.nil? || current_state.next_state_ids.blank?
                   states.where(incoming_state_ids: [])
                 else
                   states.where("$or": [{ incoming_state_ids: current_state.id }, { incoming_state_ids: [] }])
                 end

        states.to_a
      end

      def next_states(current_state)
        return [] if current_state.nil? || current_state.next_state_ids.blank?

        default_cycle = Cycle.find_by(key: 'hubdoc')
        State.where(
          cycle_id: default_cycle.id,
          :id.in => current_state.next_state_ids
        ).order_by(position: :asc).to_a
      end

      def default_next_states
        default_cycle = Cycle.find_by(key: 'hubdoc')
        State.where(
          cycle_id: default_cycle.id,
          :id.nin => default_cycle.initial_state_ids,
          incoming_state_ids: []
        ).order_by(position: :asc)
      end

      def can_change_to_state?(model, state_key, user)
        return false unless model.is_a?(Tagset)
        return true if state_key.to_sym == :share

        user.admin? || model.is_accounted?(user.id) || model.writers?(user) || model.responsible?(user)
      end
    end

    def initialize
      @cycle = nil
      @states = []
      @triggers = []
      @transitions = []
      @slinks = []
      @json_data = nil
    end

    def import_cycle_from_json
      @json_data = JSON.parse(File.read(SpiritHub::Engine.root.join('scripts', 'cycle_hub_doc.json')))

      @cycle = create_or_update_cycle(@json_data['cycle'])

      @json_data['states'].each do |state_json|
        @states << create_or_update_state(state_json)
      end

      @cycle.initial_state_ids = [State.find_by(key: 'to_classify').id]
      @cycle.save!

      @json_data['triggers'].each do |state_json|
        @triggers << create_or_update_trigger(state_json)
      end

      @json_data['transitions'].each do |state_json|
        @transitions << create_or_update_transition(state_json)
      end

      @json_data['slinks'].each do |state_json|
        @slinks << create_or_update_slinks(state_json)
      end

      remove_all_previous_models

      true
    end

    private

    def remove_all_previous_models
      State.where(cycle_id: @cycle.id, :id.nin => @states.map(&:id)).destroy_all
      Trigger.where(cycle_id: @cycle.id, :id.nin => @triggers.map(&:id)).destroy_all
      Transition.where(cycle_id: @cycle.id, :id.nin => @transitions.map(&:id)).destroy_all
      Slink.where(cycle_id: @cycle.id, :id.nin => @slinks.map(&:id)).destroy_all
    end

    def create_or_update_cycle(cycle_json)
      cycle_attibutes = cycle_json['data']['attributes']
      cycle_attibutes.delete('readers')
      cycle_attibutes.delete('writers')
      cycle_attibutes.delete('responsible')
      cycle_attibutes.delete('initial_state_ids')

      cycle = Cycle.find_or_initialize_by(key: cycle_attibutes['key'])
      cycle.assign_attributes(cycle_attibutes)
      cycle.save!
      cycle
    end

    def create_or_update_state(state_json)
      state_attibutes = state_json['data']['attributes']
      state_attibutes['next_state_ids'] = state_attibutes['next_state_ids'].map { |s| find_state_by_id(s) } if state_attibutes['next_state_ids'].present?
      state_attibutes['cycle_id'] = @cycle.id
      state_attibutes.delete('incoming_state_ids')

      state = State.find_or_initialize_by(key: state_attibutes['key'])
      state.assign_attributes(state_attibutes)
      state.save!
      state
    end

    def find_state_by_id(id)
      state_json = @json_data['states'].find { |s| s['data']['id'] == id }
      state = State.find_by(key: state_json['data']['attributes']['key'])
      return state.id if state.present?

      state = create_or_update_state(state_json)
      state.id
    end

    def create_or_update_trigger(trigger_json)
      trigger_attibutes = trigger_json['data']['attributes']
      trigger_attibutes['cycle_id'] = @cycle.id

      trigger = Trigger.find_or_initialize_by(title: trigger_attibutes['title'])
      trigger.assign_attributes(trigger_attibutes)
      trigger.save!
      trigger
    end

    def create_or_update_transition(transition_json)
      transition_attibutes = transition_json['data']['attributes']
      transition_attibutes['trigger_ids'] = transition_attibutes['trigger_ids'].map { |id| find_trigger_by_id(id) }.compact
      transition_attibutes['from_state_id'] = find_state_by_id(transition_attibutes['from_state_id']) unless transition_attibutes['from_state_id'] == 'all'
      transition_attibutes['to_state_id'] = find_state_by_id(transition_attibutes['to_state_id']) unless transition_attibutes['to_state_id'] == 'all'
      transition_attibutes['cycle_id'] = @cycle.id

      transition = Transition.find_or_initialize_by(from_state_id: transition_attibutes['from_state_id'], to_state_id: transition_attibutes['to_state_id'])
      transition.assign_attributes(transition_attibutes)
      transition.save!
      transition
    end

    def find_trigger_by_id(id)
      trigger_json = @json_data['triggers'].find { |t| t['data']['id'] == id }
      return unless trigger_json

      trigger = Trigger.find_by(title: trigger_json['data']['attributes']['title'])
      trigger&.id
    end

    def create_or_update_slinks(slink_json)
      slink_attibutes = slink_json['data']['attributes']
      slink_attibutes['from_state_id'] = find_state_by_id(slink_attibutes['from_state_id'])
      slink_attibutes['to_state_id'] = find_state_by_id(slink_attibutes['to_state_id'])
      slink_attibutes['cycle_id'] = @cycle.id

      slink = Slink.find_or_initialize_by(from_state_id: slink_attibutes['from_state_id'], to_state_id: slink_attibutes['to_state_id'])
      slink.assign_attributes(slink_attibutes)
      slink.save!
      slink
    end
  end
end
