# frozen_string_literal: true

module SpiritHub
  # Notifies users to sign a particular tagset doc
  class TagsetRequestSignature
    class << self
      def call(...)
        new(...).call
      end

      def user_candidates_to_select
        User.all.order(first_name: :asc).map { |u| ["#{u.full_name} (#{u.email})", u.id] }
      end
    end

    def initialize(options)
      @tagset = options[:tagset]
      @doc_to_sign = @tagset.docs.find_by(id: options[:doc_id])
      @recipients = Array(options[:recipients]).map(&:presence).compact
      @comments = options[:comments]
      @current_user = options[:current_user]
    end

    def call
      return false if doc_to_sign.nil?

      change_state
      change_accounted_users

      tagset.save.tap do |success|
        send_signature_request_async if success
        create_log if success
      end
    end

    private

    attr_reader :tagset, :doc_to_sign, :recipients, :comments, :current_user

    def users
      # NOTE: for now, the recipients are only user ids. Later, we'll handle groups and contacts as well.
      @users ||= User.where(:id.in => Array(recipients))
    end

    def change_state
      SpiritHub::HubdocCycle.change_state(tagset, :to_sign, current_user)
    end

    def change_accounted_users
      tagset.accounted = (users.map(&:id) + tagset.accounted.to_a).uniq
    end

    def send_signature_request_async
      doc_ids = [doc_to_sign.id.to_s]
      user_ids = users.map(&:id).map(&:to_s)

      Docs::RequestSignatureJob.perform_later(doc_ids, user_ids, current_user.id.to_s)
    end

    def create_log
      SpiritHub::JournalEntry.sent_to_action_log(
        'DOC_SENT_TO_SIGN',
        tagset:,
        user: current_user,
        recipients:,
        action_msg: ("Commentaire: #{comments}" if comments.present?)
      )
    end
  end
end
