# frozen_string_literal: true

module SpiritHub
  class TagsetSendToUsers
    VALID_STATES = %i[to_validate to_share to_complete to_answer validate refuse].freeze

    class << self
      def call(...)
        new(...).call
      end
    end

    def initialize(options)
      @state_key = validate_state_key(options[:state_key])
      @tagset = options[:tagset]
      @current_user = options[:current_user]
      @recipients = options[:recipients].to_a.map(&:to_s)
      @message = options[:message]
    end

    def call
      change_state
      store_accounted_users
      tagset.save

      create_log
    end

    private

    attr_reader :state_key, :tagset, :current_user, :recipients, :message

    def validate_state_key(state_key)
      raise ArgumentError, 'State key is required' if state_key.nil?
      raise ArgumentError, "Invalid state_key: #{state_key}. Must be one of #{VALID_STATES.join(', ')}" unless VALID_STATES.include?(state_key.to_sym)

      state_key.to_sym
    end

    def change_state
      SpiritHub::HubdocCycle.change_state(tagset, state_key, current_user)
    end

    def store_accounted_users
      tagset.accounted = (recipients + tagset.accounted.to_a).uniq
    end

    def create_log
      log_key = case state_key
                when :validate
                  'DOC_VALIDATED'
                when :refuse
                  'DOC_REFUSED'
                else
                  "DOC_SENT_#{state_key.to_s.upcase}"
                end
      SpiritHub::JournalEntry.sent_to_action_log(log_key, tagset_id: tagset.id, user_id: current_user.id, recipients:, action_msg: message)
    end
  end
end
