# frozen_string_literal: true

module SpiritHub
  # Shares a tagset doc with multiple users.
  # This module is also used for "Respond" action
  class TagsetShare
    include SpiritHub::Engine.routes.url_helpers

    CONTACT_PREFIX = 'c_'
    GROUP_PREFIX = 'g_'

    class << self
      def call(...)
        new(...).call
      end
    end

    def initialize(options)
      @tagset = options[:tagset]
      @to = options[:to]
      @cc = options[:cc]
      @bcc = options[:bcc]
      @subject = options[:subject]
      @message = options[:message]
      @attachments = options[:attachments] || []
      @attached_tagset_ids = options[:attached_tagset_ids] || []
      @current_user = options[:current_user]
      @action = options[:action].presence || :share
    end

    def call
      persist_attached_docs
      create_upload

      SpiritHub::HubdocCycle.change_state(tagset, action, current_user)

      if action == :share
        tagset.update!(
          informed_main: new_informed_main_list,
          informed_cc: new_informed_cc_list,
          informed_bcc: new_informed_bcc_list
        )
      end

      create_and_deliver_email
      log_action

      true
    end

    private

    attr_reader(
      :tagset,
      :to,
      :cc,
      :bcc,
      :subject,
      :message,
      :attachments,
      :current_user,
      :action,
      :upload
    )

    def persist_attached_docs
      attached_docs_as_tagsets = Tagset.create_tagset_docs_from_files(
        attachments,
        current_user,
        parents: tagset.parents,
        state: :done
      )

      attachment_tagsets.concat(attached_docs_as_tagsets)
    end

    def attachment_tagsets
      @attachment_tagsets ||= Tagset.where(type: 'document', :id.in => @attached_tagset_ids).to_a if @attached_tagset_ids.present?
      @attachment_tagsets ||= []
    end

    def create_upload
      @upload = Upload.create(doc_ids: shared_doc_ids, user_id: current_user.id)

      return unless action == :share

      SpiritHub::JournalEntry.share_action_log(
        'SHARE_LINK_CREATED',
        tagset:,
        user: current_user,
        upload_id: @upload.id,
        link: spirit_hub_upload_url(host: APP_CONFIG['app_host'], id: @upload.id)
      )
    end

    def shared_doc_ids
      doc_ids = tagset.doc_ids
      doc_ids += attachment_tagsets.map(&:doc_ids).flatten
      doc_ids
    end

    def create_doc_email_record(email)
      Doc.where(type: 'email').create!(
        user: current_user,
        title: email.subject,
        body: email.body,
        superfields: {
          'mail_from' => email.from.join(','),
          'mail_to' => email.to.join(','),
          'mail_cc' => email.cc.join(','),
          'mail_bcc' => email.bcc.join(',')
        }
      )
    end

    def create_and_deliver_email
      email = SpiritHub::TagsetsMailer.share(
        upload:,
        subject:,
        message:,
        to: to_emails,
        cc: cc_emails,
        bcc: bcc_emails
      )

      create_doc_email_record(email)

      email.deliver
    end

    def to_emails
      users_main.map(&:email) + contacts_main.map(&:title)
    end

    def cc_emails
      users_cc.map(&:email) + contacts_cc.map(&:title)
    end

    def bcc_emails
      users_bcc.map(&:email) + contacts_bcc.map(&:title)
    end

    def users_main
      @users_main ||= User.where(:id.in => expand_user_ids_from(to))
    end

    def users_cc
      @users_cc ||= User.where(:id.in => expand_user_ids_from(cc))
    end

    def users_bcc
      @users_bcc ||= User.where(:id.in => expand_user_ids_from(bcc))
    end

    def expand_user_ids_from(target_ids)
      target_ids.to_a.flat_map do |target_id|
        if target_id.starts_with?(GROUP_PREFIX)
          group_id = target_id.remove(GROUP_PREFIX)
          Group.find(group_id).users.pluck(:id)
        else
          target_id
        end
      end
    end

    def contacts_main
      @contacts_main ||= existing_contacts_in(to) + new_contacts_in(to)
    end

    def contacts_cc
      @contacts_cc ||= existing_contacts_in(cc) + new_contacts_in(cc)
    end

    def contacts_bcc
      @contacts_bcc ||= existing_contacts_in(bcc) + new_contacts_in(bcc)
    end

    def existing_contacts_in(target_ids)
      Doc.where(type: 'contact', :id.in => expand_contact_ids_from(target_ids))
    end

    def new_contacts_in(target_ids)
      email_addresses = target_ids.select { EmailAddress.valid?(_1) }
      email_addresses.uniq.map do |email_address|
        contact = Doc.create!(
          type: 'contact',
          title: email_address,
          superfields: { 'email_address_1' => email_address }
        )
        contact.current_user_id = current_user.id
        contact.save!
        contact
      end
    end

    def expand_contact_ids_from(target_ids)
      target_ids
        .select { _1.starts_with?(CONTACT_PREFIX) && !_1.include?('@') }
        .map { _1.remove(CONTACT_PREFIX) }
    end

    def new_informed_main_list
      new_ids = users_main.map(&:id) + contacts_main.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_main) + new_ids).uniq
    end

    def new_informed_cc_list
      new_ids = users_cc.map(&:id) + contacts_cc.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_cc) + new_ids).uniq
    end

    def new_informed_bcc_list
      new_ids = users_bcc.map(&:id) + contacts_bcc.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_bcc) + new_ids).uniq
    end

    def log_action
      case action
      when :share
        recipients = (to + cc + bcc)
        log_code = tagset.multiple_doc? ? 'MULTIPLE_DOC_SHARED' : 'DOC_SHARED'
        SpiritHub::JournalEntry.share_action_log(log_code, tagset:, user: current_user, recipients:)
      when :respond
        SpiritHub::JournalEntry.process_action_log('DOC_RESPONDED', tagset:, user: current_user)
      end
    end
  end
end
