# frozen_string_literal: true

module SpiritHub
  module Scripts
    class CreateUsersGroups
      def self.run_script
        new.create_users(Doc.find_by(title: 'entry_file_create_users_groups.csv'))
      end

      def create_csv(csv_name, rows, id_tagset)
        support = User.find_by(email: 'support@plugandwork.fr')
        csv_content = CSV.generate(col_sep: ';', row_sep: "\n", encoding: Encoding::UTF_8) do |csv|
          csv << %w[email trusted user_created groups groups_existing groups_created]
          rows.each { |row| csv << row }
        end
        dcsv = Doc.new(responsible: support.id, title: "#{csv_name}.csv", type: 'file', tagset_ids: [id_tagset])
        dcsv.blob = csv_content
        if (existing_doc = Doc.find_by(:file_md5 => dcsv.file_md5, :id.ne => dcsv.id, :version_of => nil))
          dcsv.set(current_user_id: support.id)
          dcsv.destroy
          dcsv = existing_doc
          dcsv.update(title: "#{csv_name}.csv")
        else
          dcsv.save
        end
        puts "CSV created: #{dcsv.title} - #{dcsv.id}"
      end

      def find_or_create_user(row, csv_line)
        username = row['Identifiant'].to_s.strip
        lastname = row['Nom'].to_s.strip
        firstname = row['Prénom'].to_s.strip
        email = row['mail'].to_s.downcase.strip
        is_member = row['userType'].to_s.strip
        trusted = is_member.to_s.downcase.include?('member')

        csv_line.push(email, trusted)

        if (user = User.find_by(email:))
          user.update!(login: username, first_name: firstname, last_name: lastname, trusted:)
          csv_line.push("user #{email} already exists")
        else
          user = User.create!(login: username, first_name: firstname, last_name: lastname, email:, trusted:, password: PasswordGenerator.new.password)
          csv_line.push('user created')
        end
        user
      end

      def add_group_to_user(user, row, csv_line)
        group_names = row['Groupes'].to_s.split('/').map(&:strip).reject(&:blank?)

        group_ids = []
        groups_existing = []
        groups_created = []

        group_names.each do |group_name|
          if (group = Group.find_by(name: group_name))
            groups_existing.push(group_name)
          else
            group = Group.new(name: group_name.strip)
            group.save
            groups_created.push(group_name)
          end
          group_ids.push(group.id)
        end

        user.update!(group_ids: (user.group_ids.to_a + group_ids).uniq)

        csv_line.push("#{groups_existing.length} existing groups (#{groups_existing.join(',')}) added to user") if groups_existing.length.positive?
        csv_line.push("#{groups_created.length} new groups (#{groups_created.join(',')}) added to user") if groups_created.length.positive?
      end

      def create_users(entry_csv)
        csv = CSV.parse(entry_csv.data, headers: true, liberal_parsing: true, col_sep: ';', encoding: Encoding::UTF_8)
        row_csv = []
        csv.each do |row|
          csv_line = []
          user = find_or_create_user(row, csv_line)
          add_group_to_user(user, row, csv_line)
          row_csv.push(csv_line)
        rescue StandardError => e
          puts e.class
          csv_line = ['error', e.message, row.to_s]
          row_csv.push(csv_line)
        end
        tagset_batch = Tagset.find_by(title: "Création des utilisateurs et groupes à partir d'un fichier")
        create_csv("Rapport des utilisateurs - groupes créés #{Time.now}", row_csv, tagset_batch.id)
      rescue StandardError => e
        puts e.class
        puts e.message
        puts e.backtrace if e&.backtrace
      end
    end
  end
end
