# frozen_string_literal: true

require 'faker'

FactoryBot.define do
  factory :tagset_directory, class: 'Tagset' do
    sequence(:title) { |n| "Tagset Title #{n}" }
    last_current_user_name { %w[Jean Paul Jacques Sophie].sample }
    start_date { Date.today - rand(1..30) }
    end_date { Date.today + rand(1..30) }

    type { 'directory' }
    operation_id { '123456' }
    cycle { SpiritHub::HubdocCycle.default_cycle }
    spirit_hubdoc { true }

    trait :with_journal_entries do
      after(:create) do |tagset|
        SpiritHub::JournalEntry.create_action_log('DOC_CREATED', tagset:, user_id: tagset.responsible)
        20.times do
          SpiritHub::JournalEntry.edit_action_log('DIRECTORY_INFORMATION_UPDATED', tagset:, user_id: tagset.responsible)
        end
      end
    end
  end

  factory :tagset_doc, class: 'Tagset' do
    sequence(:title) { |n| "Doc Title #{n}" }
    end_date { Date.today + rand(1..30) }
    type { 'document' }
    operation_id { '123456' }
    cycle { SpiritHub::HubdocCycle.default_cycle }
    spirit_hubdoc { true }

    trait :with_parents do
      after(:create) do |tagset|
        tagset_dir1 = create(:tagset_directory)
        tagset_dir1.update!(tagset_ids: [tagset.id.to_s])
        tagset_dir2 = create(:tagset_directory)
        tagset_dir2.update!(tagset_ids: [tagset.id.to_s])
      end
    end

    trait :with_files do
      transient do
        filenames { ['dog.jpg'] }
      end

      after(:create) do |tagset, evaluator|
        uploaded_files = evaluator.filenames.map do |filename|
          file_path = Rails.root.join("packages/spirit_hub/spec/fixtures/files/#{filename}")

          ActionDispatch::Http::UploadedFile.new(
            tempfile: File.open(file_path, 'rb'),
            filename: File.basename(file_path),
            type: 'application/pdf'
          )
        end

        tagset.add_doc_to_tagset_doc(uploaded_files, tagset.responsible && User.find(tagset.responsible))

        DocPreviewWorker.new.perform(tagset.doc_ids.last)
        tagset.save
      end
    end

    trait :with_file do
      with_files
      filenames { [filename] }

      transient do
        filename { 'dog.jpg' }
      end
    end

    trait :with_informed do
      after(:create) do |tagset|
        users = FactoryBot.create_list(:user, rand(1..5))
        tagset.update(informed: users.map(&:id))
      end
    end

    trait :with_journal_entries do
      after(:create) do |tagset|
        SpiritHub::JournalEntry.create_action_log('DOC_CREATED', tagset:, user_id: tagset.responsible)
        20.times do
          SpiritHub::JournalEntry.edit_action_log('DIRECTORY_INFORMATION_UPDATED', tagset:, user_id: tagset.responsible)
        end
      end
    end

    trait :with_state do
      transient do
        state_key { :classify }
      end
      after(:create) do |tagset, evaluator|
        SpiritHub::HubdocCycle.change_state(tagset, evaluator.state_key, User.all.sample)
        tagset.save
      end
    end
  end
end
