# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Operation, type: :model) do
  it 'has a valid factory' do
    expect(true).to be_truthy
  end

  describe 'validations' do
    subject { build(:operation) }
    it { is_expected.to validate_presence_of(:code) }
    it { is_expected.to validate_presence_of(:title) }
    it { is_expected.to validate_presence_of(:city) }
    it { is_expected.to validate_presence_of(:address) }
    it { is_expected.to validate_presence_of(:postal_code) }
    it { is_expected.to validate_presence_of(:agency) }
    it { is_expected.to validate_presence_of(:type) }
    it { is_expected.to validate_presence_of(:start_date) }
    it { is_expected.to validate_presence_of(:end_date) }

    it 'custom code taken error message' do
      create(:operation, code: 'test')
      operation = subject
      operation.set(code: 'test')
      operation.valid?
      expect(operation.errors[:code]).to include('est déjà pris')
    end

    context 'when validating date order' do
      let(:operation) { build(:operation, start_date: Date.today, end_date: Date.yesterday) }

      it 'is invalid if end_date is before start_date' do
        operation.valid?
        expect(operation.errors[:end_date]).to include('doit être postérieure à la date de début')
      end
    end
  end

  describe 'code' do
    let!(:operation) { create(:operation, code: 'test') }

    it 'should upcase the code' do
      expect(operation.reload.code).to eq('TEST')
    end

    it 'should be unique and case insensitive' do
      expect(build(:operation, code: 'test')).to_not be_valid
      expect(build(:operation, code: 'TEST')).to_not be_valid
    end
  end

  describe 'updated_tagset_operation_title' do
    let!(:operation) { create(:operation, :with_tagset_root) }
    let!(:tagset) { create(:tagset, operation:) }

    it 'should update the title of the root tagset' do
      operation.update(title: 'new title')
      expect(operation.root_tagset.reload.title).to eq('Projet new title')
    end

    it 'should not update the operation_title of all tagset' do
      operation.update(title: 'new title')
      expect(tagset.reload.operation_title).to eq('new title')
    end
  end

  describe 'update_directories' do
    let!(:operation) { create(:operation) }
    let!(:tagset_directory) { create(:tagset_directory, operation:) }
    let!(:tagset_doc) { create(:tagset_doc, operation:) }

    RSpec.shared_examples 'update directories attributes' do |attribute, type|
      it "#{attribute} with the right values" do
        old_value = tagset_doc.send(attribute)
        new_value = case type.to_sym
                    when :date
                      date = Date.today
                      date -= 1.month if attribute == :start_date
                      date += 1.month if attribute == :end_date
                      date
                    when :user
                      create(:user, first_name: 'Jean', last_name: 'Dupont').id
                    end

        operation.update(attribute => new_value)
        expect(tagset_directory.reload.send(attribute)).to eq(new_value)
        expect(tagset_doc.reload.send(attribute)).to eq(old_value)
      end
    end

    include_examples 'update directories attributes', :start_date, :date
    include_examples 'update directories attributes', :end_date, :date
    include_examples 'update directories attributes', :responsible, :user
    include_examples 'update directories attributes', :current_user_id, :user
  end

  describe 'destroy_tagsets' do
    let!(:operation) { create(:operation) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file, operation:) }
    let!(:tagset_directory) { create(:tagset_directory, operation:) }

    it 'should destroy all tagsets' do
      operation.destroy
      expect(Tagset.where(operation_id: operation.id)).to be_empty
    end
  end

  describe 'generate_operation_tree' do
    let!(:doc_csv) { create(:doc, title: 'doc.csv', file_path: 'packages/spirit_hub/spec/fixtures/operation_trees/sample1.csv') }
    let!(:operation) { create(:operation, title: 'Operation 1', classement_plan_id: doc_csv.id) }
    let!(:user) { create(:user) }

    subject { operation.generate_operation_tree(user) }

    it 'should create all groups' do
      subject

      expect(Group.all.pluck(:name)).to match_array(%w[spirit-immo_alsace_responsable-conformité
                                                       spirit-immo_alsace_responsable-juridique spirit-immo_alsace_responsable-maitrise
                                                       spirit-immo_alsace_responsable-terrain spirit-immo_alsace_service_juridique])
    end

    it 'should create all phases' do
      subject
      expect(operation.root_tagset.title).to eq('Projet Operation 1')
      expect(SpiritHub::Phase.where(operation_id: operation.id).pluck(:title)).to match_array(['Etude', 'Initialisation projet', 'Suivi projet', 'Travaux'])
      expect(SpiritHub::Phase.find_by(operation_id: operation.id, title: 'Etude').parent.title).to eq('Initialisation projet')
      expect(SpiritHub::Phase.find_by(operation_id: operation.id, title: 'Etude').initial_date).to eq(operation.start_date + 4.months)
    end

    it 'directory with the right attributes' do
      subject

      terrain = Tagset.find_by(operation:, title: 'Terrain')
      expect(Tagset.where(:id.in => terrain.tagset_ids).pluck(:title)).to match_array(['Descriptif du foncier', 'Acquisition Propriétaire XX', 'Etudes réalisées'])
      expect(terrain.phase.title).to eq('Initialisation projet')
      expect(terrain.responsible).to eq(operation.responsible)
      expect(terrain.current_user_id).to eq(operation.current_user_id)
      expect(terrain.accounted).to eq([operation.current_user_id])
      expect(terrain.start_date).to eq(operation.start_date)
      expect(terrain.end_date).to eq(operation.end_date)
      expect(terrain.superfields[:chapter]).to eq('Opération de promotion')
      expect(terrain.superfields[:sub_chapter]).to eq('3 - MONTAGE')
      expect(terrain.max_duplicates).to be_nil
      terrain_reader_groups = terrain.readers.map { |a| a.gsub('g_', '') }
      terrain_writer_groups = terrain.writers.map { |a| a.gsub('g_', '') }
      expect(Group.where(:id.in => terrain_reader_groups).pluck(:name)).to match_array(['spirit-immo_alsace_responsable-terrain'])
      expect(Group.where(:id.in => terrain_writer_groups).pluck(:name)).to match_array(%w[spirit-immo_alsace_responsable-maitrise spirit-immo_alsace_responsable-terrain])
      first_completeness = SpiritHub::Completeness.find_by(tagset_id: terrain.id)
      expect(first_completeness.title).to eq('Convention(s) collectivités')
      expect(first_completeness.superfields[:document_type]).to eq('1.25')

      etude_realisees = Tagset.find_by(operation:, title: 'Etudes réalisées')
      expect(etude_realisees.phase.title).to eq('Etude')
      expect(etude_realisees.responsible).to eq(operation.responsible)
      expect(etude_realisees.current_user_id).to eq(operation.current_user_id)
      expect(etude_realisees.start_date).to eq(operation.start_date)
      expect(etude_realisees.end_date).to eq(operation.end_date)
      expect(etude_realisees.superfields[:chapter]).to eq('Opération de promotion')
      expect(etude_realisees.superfields[:sub_chapter]).to eq('9 - MONTAGE')
      expect(etude_realisees.max_duplicates).to eq('*')
      etude_realisees_reader_groups = etude_realisees.readers.map { |a| a.gsub('g_', '') }
      etude_realisees_writer_groups = etude_realisees.writers.map { |a| a.gsub('g_', '') }
      expect(Group.where(:id.in => etude_realisees_reader_groups).pluck(:name)).to be_empty
      expect(Group.where(:id.in => etude_realisees_writer_groups).pluck(:name)).to match_array(%w[spirit-immo_alsace_responsable-maitrise spirit-immo_alsace_responsable-terrain])
      first_completeness = SpiritHub::Completeness.find_by(tagset_id: etude_realisees.id)
      expect(first_completeness.title).to eq('Etude des sols préliminaire')
      expect(first_completeness.superfields[:document_type]).to eq('1.1')
    end

    it 'add journal entries' do
      allow(SpiritHub::JournalEntry).to receive(:create)

      tagset_root = subject

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'OPERATION_CREATED',
        user_id: user.id,
        tagset_id: tagset_root.id,
        action: "Projet créé par #{user.full_name}"
      ).once

      Tagset.where(operation_id: operation.id).each do |tagset|
        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DIRECTORY_CREATED',
          user_id: user.id,
          tagset_id: tagset.id,
          action: "Dossier créé par #{user.full_name}"
        ).once
      end
    end
  end
end
