# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::OperationPolicy do
  let(:admin) { create(:user, roles: ['Administrator']) }
  let(:responsible) { create(:user) }
  let(:reader) { create(:user) }
  let(:writer) { create(:user) }
  let(:other_user) { create(:user) }
  let(:operation) { create(:operation, responsible: responsible.id, readers: [reader.id], writers: [writer.id, responsible.id]) }

  subject { described_class }

  permissions :index?, :sort? do
    it 'permits all users' do
      [admin, responsible, writer, reader, other_user].each do |user|
        expect(subject).to permit(user)
      end
    end
  end

  permissions :update?, :edit? do
    it 'permits admins' do
      expect(subject).to permit(admin, operation)
    end

    it 'permits writers of the operation' do
      [responsible, writer].each do |user|
        expect(subject).to permit(user, operation)
      end
    end

    it 'denies access to other users' do
      [reader, other_user].each do |user|
        expect(subject).not_to permit(user, operation)
      end
    end
  end

  permissions :create?, :new? do
    it 'permits all users' do
      [admin, responsible, writer, reader, other_user].each do |user|
        expect(subject).to permit(user, operation)
      end
    end
  end

  permissions :destroy? do
    it 'permits admins' do
      expect(subject).to permit(admin, operation)
    end

    it 'permits the responsible user' do
      expect(subject).to permit(responsible, operation)
    end

    it 'denies access to other users' do
      [writer, reader, other_user].each do |user|
        expect(subject).not_to permit(user, operation)
      end
    end
  end

  describe SpiritHub::OperationPolicy::Scope do
    let!(:operation1) { create(:operation, readers: [reader.id]) }
    let!(:operation2) { create(:operation) }
    let(:scope) { SpiritHub::Operation.all }

    subject { described_class.new(user, scope).resolve }

    context 'when the user is an admin' do
      let(:user) { admin }

      it 'returns all operations' do
        expect(subject).to match_array([operation1, operation2])
      end
    end

    context 'when the user is a reader' do
      let(:user) { reader }

      it 'returns operations readable by the user' do
        expect(subject).to match_array([operation1])
      end
    end

    context 'when other_user' do
      let(:user) { other_user }

      it 'returns operations readable by the user' do
        expect(subject).to match_array([])
      end
    end
  end
end
