# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetShare, type: :service do
  subject { described_class.call(**params) }

  let(:tagset) { create(:tagset_doc, :with_file) }

  let(:user1) { create(:user) }
  let(:user2) { create(:user) }
  let(:user3) { create(:user) }
  let(:user4) { create(:user) }

  let(:message_subject) { Faker::Lorem.sentence }
  let(:message) { Faker::Lorem.sentence }

  before { SpiritHub::HubdocCycle.import_cycle_from_json }
  before { allow(SpiritHub::TagsetsMailer).to receive(:share).and_call_original }
  before { allow(SpiritHub::JournalEntry).to receive(:create) }
  before { allow(LogJob).to receive(:perform_async) }

  context 'when share action' do
    let(:params) do
      {
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      }
    end

    it 'shares the doc to the user' do
      Upload.destroy_all
      Doc.where(type: 'email').destroy_all
      allow(SpiritHub::JournalEntry).to receive(:create)

      subject

      # creates an Upload record
      expect(upload_created = Upload.last).to be_present
      expect(upload_created.user_id).to eq user4.id
      expect(upload_created.doc_ids).to be_present
      expect(upload_created.doc_ids).to match_array tagset.doc_ids
      expect(upload_created.doc_id).to be_nil

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id])
      expect(tagset.informed_main).to match_array([user1.id])
      expect(tagset.informed_cc).to match_array([user2.id])
      expect(tagset.informed_bcc).to match_array([user3.id])
      expect(tagset.current_state).to eq('Partagé')
      expect(tagset.state.key).to eq('share')

      # creates the Doc(type=email) record
      expect(doc_email = Doc.where(type: 'email').last).to be_present
      expect(doc_email.user).to eq user4
      expect(doc_email.title).to eq message_subject
      expect(doc_email.body).to include message
      expect(doc_email.superfields['mail_from']).to eq user4.email
      expect(doc_email.superfields['mail_to']).to eq user1.email
      expect(doc_email.superfields['mail_cc']).to eq user2.email
      expect(doc_email.superfields['mail_bcc']).to eq user3.email

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        upload: upload_created,
        subject: message_subject,
        message:,
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )

      # journal_entries
      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_SHARED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Document #{tagset.title} partagé par #{user4.full_name} à #{user1.full_name}, #{user2.full_name} et #{user3.full_name}"
      ).once

      upload = Upload.last
      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'SHARE_LINK_CREATED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Lien de partage: http://#{APP_CONFIG['app_host']}/spirit_hub/uploads/#{upload.id}"
      ).once
    end

    it 'delivers the share email' do
      share_email = double(
        deliver: nil,
        subject: message_subject,
        body: message,
        from: [user4.email],
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )
      allow(SpiritHub::TagsetsMailer).to receive(:share).and_return(share_email)

      subject

      expect(share_email).to have_received(:deliver)
    end

    context 'when passing a group as one of the main recipients' do
      before { params[:to] += [group_id_prefixed] }

      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_main).to match_array([user1.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, user5.email, user6.email]
          )
        )
      end
    end

    context 'when passing a group as one of the CC recipients' do
      before { params[:cc] += [group_id_prefixed] }

      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_cc).to match_array([user2.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, user5.email, user6.email]
          )
        )
      end
    end

    context 'when passing a group as one of the BCC recipients' do
      before { params[:bcc] += [group_id_prefixed] }

      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_bcc).to match_array([user3.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, user5.email, user6.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the main recipients' do
      before { params[:to] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, contact.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the CC recipients' do
      before { params[:cc] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, contact.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the BCC recipients' do
      before { params[:bcc] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, contact.email]
          )
        )
      end
    end

    context 'when passing a NEW contact as one of the main recipients' do
      before { params[:to] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, contact_created.email]
          )
        )
      end
    end

    context 'when passing a NEW contact as one of the CC recipients' do
      before { params[:cc] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, contact_created.email]
          )
        )
      end
    end

    context 'when passing a NEW contact as one of the BCC recipients' do
      before { params[:bcc] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, contact_created.email]
          )
        )
      end
    end

    context 'when passing attached files' do
      let(:attached_files) do
        [
          double('file1', original_filename: 'document1.pdf', read: 'File content 1'),
          double('file2', original_filename: 'document2.pdf', read: 'File content 2')
        ]
      end

      let(:tagset) { create(:tagset_doc, :with_file, :with_parents) }
      let(:parents_from_original_shared_tagset_doc) { tagset.reload.parents }
      let(:attached_tagset_doc1) { create(:tagset_doc, :with_file, :with_parents) }
      let(:attached_tagset_doc2) { create(:tagset_doc, :with_file, :with_parents) }

      before do
        params[:attachments] = attached_files
        params[:attached_tagset_ids] = [attached_tagset_doc1.id, attached_tagset_doc2.id]
      end

      it 'captures each of the attached files as a tagset doc with the correct state' do
        allow(Doc).to receive(:create_from_file!).and_call_original

        expect(parents_from_original_shared_tagset_doc).to be_present

        subject

        expect(Doc).to have_received(:create_from_file!).with(attached_files[0], user4)
        expect(Doc).to have_received(:create_from_file!).with(attached_files[1], user4)

        attached_files.each do |file|
          tagset_doc = Tagset.find_by(title: file.original_filename)
          expect(tagset_doc).to be_present
          expect(tagset_doc.type).to eq 'document'
          expect(tagset_doc.docs.length).to eq 1
          expect(tagset_doc.docs.first.title).to eq file.original_filename
          expect(tagset_doc.parents).to be_present
          expect(tagset_doc.parents).to match_array parents_from_original_shared_tagset_doc
          expect(tagset_doc.cycle_id).to eq SpiritHub::HubdocCycle.default_cycle.id
          expect(tagset_doc.state.key).to eq('done')
        end
      end

      it 'does not update any of the existing attached tagset docs attributes' do
        expect(parents_from_original_shared_tagset_doc).to be_present

        subject

        expect(attached_tagset_doc1.reload.parents).to_not match_array(parents_from_original_shared_tagset_doc)
        expect(attached_tagset_doc2.reload.parents).to_not match_array(parents_from_original_shared_tagset_doc)
        expect(attached_tagset_doc1.state&.key).to_not eq('done')
        expect(attached_tagset_doc2.state&.key).to_not eq('done')
      end

      it 'shares the doc including the attachments to the user' do
        Upload.destroy_all

        subject

        attached_doc1 = attached_tagset_doc1.doc_ids
        attached_doc2 = attached_tagset_doc2.doc_ids
        attached_doc3 = Doc.find_by!(title: 'document1.pdf').id
        attached_doc4 = Doc.find_by(title: 'document2.pdf').id

        # creates an Upload record
        expect(upload_created = Upload.last).to be_present
        expect(upload_created.user_id).to eq user4.id
        expect(upload_created.doc_ids).to be_present
        expect(upload_created.doc_ids).to match_array [*tagset.doc_ids, *attached_doc1, *attached_doc2, attached_doc3, attached_doc4]
      end
    end
  end

  context 'when respond action' do
    let(:params) do
      {
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4,
        action: :respond
      }
    end

    it 'shares the doc to the user' do
      Upload.destroy_all
      Doc.where(type: 'email').destroy_all
      allow(SpiritHub::JournalEntry).to receive(:create)

      subject

      # creates an Upload record
      expect(upload_created = Upload.last).to be_present
      expect(upload_created.user_id).to eq user4.id
      expect(upload_created.doc_ids).to be_present
      expect(upload_created.doc_ids).to match_array tagset.doc_ids
      expect(upload_created.doc_id).to be_nil

      # creates the Doc(type=email) record
      expect(doc_email = Doc.where(type: 'email').last).to be_present
      expect(doc_email.user).to eq user4
      expect(doc_email.title).to eq message_subject
      expect(doc_email.body).to include message
      expect(doc_email.superfields['mail_from']).to eq user4.email
      expect(doc_email.superfields['mail_to']).to eq user1.email
      expect(doc_email.superfields['mail_cc']).to eq user2.email
      expect(doc_email.superfields['mail_bcc']).to eq user3.email

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        upload: upload_created,
        subject: message_subject,
        message:,
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )

      # journal_entries
      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_RESPONDED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Réponse effectuée pour #{tagset.title} par #{user4.full_name}"
      ).once
    end

    it 'delivers the share email' do
      share_email = double(
        deliver: nil,
        subject: message_subject,
        body: message,
        from: [user4.email],
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )
      allow(SpiritHub::TagsetsMailer).to receive(:share).and_return(share_email)

      subject

      expect(share_email).to have_received(:deliver)
    end
  end
end
