# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetSuggester, type: :service do
  let(:suggester) { described_class.new(tagset, user) }

  let(:tagset) { create(:tagset_doc) }
  let(:doc1) { create(:doc) }
  let(:doc2) { create(:doc) }
  let(:user) { create(:user) }

  before do
    allow(tagset).to receive(:docs).and_return([doc1, doc2])
  end

  describe '#suggest_spaces_to_classify' do
    subject { suggester.suggest_spaces_to_classify(limit: 3) }

    let(:doc1_spaces_suggestions) { [] }
    let(:doc2_spaces_suggestions) { [] }

    before do
      allow(doc1).to receive(:suggests_for).with('spaces', user:, limit: 3).and_return(doc1_spaces_suggestions)
      allow(doc2).to receive(:suggests_for).with('spaces', user:, limit: 3).and_return(doc2_spaces_suggestions)
    end

    it 'returns an empty array when nothing to suggest' do
      expect(subject).to eq []
    end

    context 'when there are space suggestions to the underlying docs' do
      let(:doc1_spaces_suggestions) { create_list(:tagset_directory, 1) }
      let(:doc2_spaces_suggestions) { create_list(:tagset_directory, 2) }

      it 'returns those as suggestions' do
        expect(subject).to match_array(doc1_spaces_suggestions + doc2_spaces_suggestions)
      end
    end

    context 'when there are space suggestions to the underlying docs, with common spaces' do
      let(:doc1_spaces_suggestions) { [create(:tagset_directory)] }
      let(:doc2_spaces_suggestions) { [doc1_spaces_suggestions.first, create(:tagset_directory)] }

      it 'returns those as suggestions, avoiding the duplicates' do
        expect(subject).to match_array [doc1_spaces_suggestions.first, doc2_spaces_suggestions.second]
      end
    end

    context 'when there are space suggestions to the underlying docs, above the limit' do
      let(:doc1_spaces_suggestions) { create_list(:tagset_directory, 2) }
      let(:doc2_spaces_suggestions) { create_list(:tagset_directory, 3) }

      it 'returns some of those as suggestions, within the limit' do
        spaces_suggestions = doc1_spaces_suggestions + doc2_spaces_suggestions

        expect((subject & spaces_suggestions).length).to eq 3
      end
    end
  end
end
